﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;

namespace Songhay.Wpf.BiggestBox.IndexPart.ViewModels
{
    using Songhay.Models;
    using Songhay.Wpf.Mvvm;
    using Songhay.Wpf.Mvvm.ViewModels;
    using Views;

    /// <summary>
    /// Models the Packed XAML samples for display
    /// </summary>
    public class PackedXamlViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PackedXamlViewModel"/> class.
        /// </summary>
        public PackedXamlViewModel()
        {
            this.Resources = ApplicationUtility
                .ListPackedResources(Assembly.GetExecutingAssembly());
        }

        /// <summary>
        /// Gets the load packed XAML command.
        /// </summary>
        /// <value>The load packed XAML command.</value>
        public ICommand LoadPackedXaml
        {
            get
            {
                if(_loadPackedXamlCommand == null)
                    _loadPackedXamlCommand = new RelayCommand<string>(param =>
                    {
                        var uri = new Uri(param, UriKind.Relative);
                        var collectionItem = this.Resources.First(r =>
                            {
                                var resourceLocation = new Uri(r.Key, UriKind.Relative);
                                return resourceLocation.Equals(uri);
                            });

                        var xaml = ApplicationUtility.LoadResource(uri);
                        var window = new PackedXamlWindow(uri, xaml);
                        window.WindowStartupLocation = WindowStartupLocation.CenterScreen;
                        window.Title = collectionItem.DisplayName;
                        window.Show();
                    });

                return _loadPackedXamlCommand;
            }
        }

        /// <summary>
        /// Gets or sets the collection of loose XAML resources.
        /// </summary>
        /// <value>The resources.</value>
        public IEnumerable<PackedResource> Resources { get; private set; }

        RelayCommand<string> _loadPackedXamlCommand;
    }
}
