﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Management;

namespace Songhay.Wpf.BiggestBox.IndexPart.ViewModels
{
    using Extensions;
    using Songhay.Models;
    using Songhay.Wpf.Mvvm.ViewModels;

    /// <summary>
    /// View Model for network device report
    /// </summary>
    public class ReportNetworkViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="ReportNetworkViewModel"/> class.
        /// </summary>
        public ReportNetworkViewModel()
        {
            this._networkDevices = new ObservableCollection<HierarchicalNameValuePair>();

            var mos = new ManagementObjectSearcher(
                @"SELECT *
                    FROM Win32_NetworkAdapter
                    WHERE Manufacturer != 'Microsoft'
                    AND NOT PNPDeviceID LIKE 'ROOT\\%'"
            );

            // Get the physical adapters and sort them by their index.
            // This is needed because they're not sorted by default
            mos.Get().OfType<ManagementObject>()
                .OrderBy(p => Convert.ToUInt32(p.Properties["Index"].Value))
                .ForEachInEnumerable(mo =>
                {
                    var device = new HierarchicalNameValuePair();
                    var list = new List<NameValuePair>();
                    device.Name = "Network Device";
                    device.Value = mo.Path.RelativePath;
                    mo.Properties
                        .OfType<PropertyData>().ForEachInEnumerable(data =>
                        {
                            var line = string.Format("{0}",
                                (data.Value ?? "N/A").ToString()
                                    .Replace("{", "{{").Replace("}", "}}"));
                            list.Add(
                                new NameValuePair
                                {
                                    Name = data.Name,
                                    Value = line
                                });
                        });
                    device.NameValuePairs = list.ToArray();
                    this._networkDevices.Add(device);
                });
        }

        /// <summary>
        /// Gets the network devices.
        /// </summary>
        /// <value>The network devices.</value>
        public ObservableCollection<HierarchicalNameValuePair> NetworkDevices
        {
            get
            {
                return this._networkDevices;
            }
        }

        ObservableCollection<HierarchicalNameValuePair> _networkDevices;
    }
}
