﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Input;
using GalaSoft.MvvmLight.Command;

namespace Songhay.Wpf.BiggestBox.IndexPart.ViewModels
{
    using Songhay.Models;
    using Songhay.Wpf.Mvvm.ViewModels;
    using Songhay.Wpf.Mvvm.Views;
    using Views.UserControls;

    /// <summary>
    /// User Control View Model.
    /// </summary>
    public class UserControlsViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance
        /// of the <see cref="UserControlsViewModel"/> class.
        /// </summary>
        public UserControlsViewModel()
        {
            this.Controls = new ObservableCollection<LazyDisplayItem<UserControl>>();

            this.Controls.Add(
                new LazyDisplayItem<UserControl>
                {
                    DisplayName = "Demonstrates how Grid children are generated dynamically using the LogicalTreeHelper.",
                    LazyItem = new Lazy<UserControl>
                        (
                            new Func<UserControl>(() => new DynamicGridSample())
                        ),
                    Name = "DynamicGridSample"
                });
            this.Controls.Add(
                new LazyDisplayItem<UserControl>
                {
                    DisplayName = "Demonstrates explicit (imperative) validation using existing XAML bindings.",
                    LazyItem = new Lazy<UserControl>
                        (
                            new Func<UserControl>(() => new ValidationSample())
                        ),
                    Name = "ValidationSample"
                });
        }

        /// <summary>
        /// Gets the load-user-control command.
        /// </summary>
        /// <value>The load-user-control command.</value>
        public ICommand LoadUserControl
        {
            get
            {
                if(_loadUserControlCommand == null)
                    _loadUserControlCommand = new RelayCommand<string>(
                    param =>
                    {
                        var control = this.Controls
                            .Where(uc => uc.Name == param)
                            .FirstOrDefault();

                        //TODO: consider using the Error child window.
                        if(control == null) throw new Exception("User Control not found.");

                        var alert = new UserControlView(control.LazyItem.Value);
                        alert.Show();
                    });

                return _loadUserControlCommand;
            }
        }

        /// <summary>
        /// Gets or sets the collection of sample user controls.
        /// </summary>
        /// <value>The controls.</value>
        public ObservableCollection<LazyDisplayItem<UserControl>> Controls { get; private set; }

        RelayCommand<string> _loadUserControlCommand;

    }
}
