﻿using System;
using System.Windows;
using System.Windows.Controls;

namespace Songhay.Wpf.BiggestBox.IndexPart.Views.UserControls
{
    using CustomControls;

    /// <summary>
    /// Dynamic Grid Sample User Control
    /// </summary>
    public partial class DynamicGridSample : UserControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DynamicGridSample"/> class.
        /// </summary>
        public DynamicGridSample()
        {
            InitializeComponent();

            this.Loaded += (s, args) => {
                var childrenCountSlider = (Slider)LogicalTreeHelper.FindLogicalNode(this, "ChildrenCountSlider");
                childrenCountSlider.ValueChanged += new RoutedPropertyChangedEventHandler<double>(OnChildrenCountChanged);

                var columnCountSlider = (Slider)LogicalTreeHelper.FindLogicalNode(this, "ColumnCountSlider");
                columnCountSlider.ValueChanged += new RoutedPropertyChangedEventHandler<double>(OncolumnCountChanged);
            };
        }

        void OnChildrenCountChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            var childrenCount = (int)Math.Floor(e.NewValue + 0.5);

            var grid = (DynamicGrid)LogicalTreeHelper.FindLogicalNode(this, "TargetGrid");
            while(grid.Children.Count < childrenCount)
            {
                var control = new Control();
                grid.Children.Add(control);
                control.Style = (Style)control.FindResource("ImageWithBorder");
            }
            while(grid.Children.Count > childrenCount)
            {
                grid.Children.Remove(grid.Children[grid.Children.Count - 1]);
            }

            var displayText = (TextBlock)LogicalTreeHelper.FindLogicalNode(this, "ChildrenCountDisplay");
            displayText.Text = childrenCount.ToString();
        }

        void OncolumnCountChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            var columnCount = (int)Math.Floor(e.NewValue + 0.5);

            var grid = (DynamicGrid)LogicalTreeHelper.FindLogicalNode(this, "TargetGrid");
            while(grid.ColumnDefinitions.Count < columnCount)
            {
                grid.ColumnDefinitions.Add(new ColumnDefinition());
            }
            while(grid.ColumnDefinitions.Count > columnCount)
            {
                grid.ColumnDefinitions.Remove(grid.ColumnDefinitions[grid.ColumnDefinitions.Count - 1]);
            }

            var displayText = (TextBlock)LogicalTreeHelper.FindLogicalNode(this, "ColumnCountDisplay");
            displayText.Text = columnCount.ToString();
        }
    }
}
