﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using GalaSoft.MvvmLight.Messaging;

namespace Songhay.Wpf.BiggestBox
{
    using Composition;
    using Extensions;
    using Songhay.Mvvm.Extensions;
    using Songhay.Mvvm.Models;
    using Songhay.Wpf.BiggestBox.Properties;
    using Songhay.Wpf.BiggestBox.Views;
    using Songhay.Wpf.Mvvm.ViewModels;
    using Songhay.Wpf.Mvvm.Views;

    /// <summary>
    /// Defines the MEF Composition Host for this application.
    /// </summary>
    internal sealed class BiggestBoxCompositionHost : CompositionHost, INavigationCompositionHost
    {
        #region INavigationCompositionHost members:

        /// <summary>
        /// The pages, hosted by the MEF Composition.
        /// </summary>
        /// <remarks>
        /// This is a public Property for Silverlight security reasons
        /// (for <c>Microsoft.Internal.ReflectionInvoke.SafeInvoke</c> in MEF).
        /// The WPF equivalent can be a private field.
        /// </remarks>
        [ImportMany(typeof(Page))]
        public IEnumerable<Page> ComposedPages { get; set; }

        /// <summary>
        /// Gets the page, hosted by the MEF Composition.
        /// </summary>
        /// <param name="typeName">
        /// Name of the type
        /// of the <see cref="System.Windows.Controls.Page"/>.
        /// </param>
        public Page GetPage(string typeName)
        {
            if(this.ComposedPages == null)
                throw new Exception("The Composed Pages are null.");

            var page = this.ComposedPages
                .FirstOrDefault(p =>
                    p.GetType().Name.Contains(typeName));

            if(page == null)
            {
                page = new ApplicationMessagePage();
                this.NavigationTarget.SetNavigationContent(page);

                var data = new ApplicationMessagePageViewModel
                {
                    ApplicationMessage = "Page, " + typeName + ", not found.",
                    IsErrorMessage = true,
                    PageTitle = "Page not found.",
                };

                var message = new LightNavigationMessage<ApplicationMessagePageViewModel>(data);
                Messenger.Default.Send(message);
            }

            return page;
        }

        /// <summary>
        /// Gets the root visual
        /// of this application.
        /// </summary>
        /// <remarks>
        /// The conventional Application Loader uses Reflection to call this method.
        /// This call usually instantiates the Host singleton.
        /// </remarks>
        public Window GetRootVisual()
        {
            var visual = new ClientView();
            return visual;
        }

        #endregion

        /// <summary>
        /// Gets the MEF Host.
        /// </summary>
        /// <value>The MEF Host.</value>
        public static BiggestBoxCompositionHost MefHost
        {
            get { return compositionHost; }
        }

        /// <summary>
        /// Gets or sets the navigation target.
        /// </summary>
        /// <value>The navigation target.</value>
        internal Frame NavigationTarget { get; set; }

        /// <summary>
        /// Initializes a new instance
        /// of the <see cref="BiggestBoxCompositionHost"/> class.
        /// </summary>
        /// <param name="pathToPages">The path to pages.</param>
        BiggestBoxCompositionHost(string pathToPages)
        {
            this.AddToAggregateCatalog(pathToPages);
            this.AddToAggregateCatalog(CompositionUtility.GetAssemblyCatalog(typeof(CompositionMessaging)));

            this.AddToCompositionBatch(this);
            base.Compose();

            if(this.CompositionMessaging != null)
                this.CompositionMessaging.ForEachInEnumerable(i =>
                {
                    i.RegisterMessageReceivers();
                    i.SendCompositionMessages();
                });
        }

        /// <summary>
        /// The singleton read-only instance.
        /// </summary>
        static readonly BiggestBoxCompositionHost compositionHost = new BiggestBoxCompositionHost(Settings.Default.CompositionWpfPagesPath);
    }
}
