﻿using System.ComponentModel.Composition;
using System.Windows.Controls;
using GalaSoft.MvvmLight.Messaging;

namespace Songhay.Wpf.BiggestBox
{
    using Songhay.Composition;
    using Songhay.Mvvm.Extensions;
    using Songhay.Mvvm.Models;
    using Songhay.Wpf.BiggestBox.Views;

    /// <summary>
    /// Defines the Composition Messaging
    /// for the Composition Host.
    /// </summary>
    [Export(typeof(ICompositionMessaging))]
    public class CompositionMessaging : ICompositionMessaging
    {
        #region ICompositionMessaging Members

        public void SendCompositionMessages()
        {
        }

        /// <summary>
        /// Registers the Composition Messaging message receivers.
        /// </summary>
        public void RegisterMessageReceivers()
        {
            this.ReceiveClientMessage();
        }

        #endregion

        void ReceiveClientMessage()
        {
            Messenger.Default.Register<LightMessage<ClientView>>(this,
                message =>
                {
                    if(message.ViewId != "ClientView") return;
                    if(message.Source != LightMessageSource.ViewEventLoaded) return;

                    this.ReceiveClientNavigationMessages(message.Content);

                    var page = BiggestBoxCompositionHost.MefHost.GetPage("IndexPage");
                    var frame = message.Content.IndexFrame;
                    BiggestBoxCompositionHost.MefHost.NavigationTarget = frame;
                    BiggestBoxCompositionHost.MefHost.NavigationTarget.SetNavigationContent(page);
                });
        }

        void ReceiveClientNavigationMessages(ClientView view)
        {
            Messenger.Default.Register<LightNavigationMessage<string>>(view,
                message =>
                {
                    if(view.IsNavigating || (message == null)) return;

                    if(message.HasWellFormedUriString)
                        BiggestBoxCompositionHost.MefHost.NavigationTarget.SetNavigationContent(message.TargetAbsoluteUri);
                    else
                    {
                        //Navigate to Visual object with Visual Type name:
                        var page = BiggestBoxCompositionHost.MefHost.GetPage(message.TargetLocation);
                        BiggestBoxCompositionHost.MefHost.NavigationTarget.SetNavigationContent(page);
                    }
                });

            Messenger.Default.Register<LightNavigationMessage<Page>>(view,
                message =>
                {
                    if(view.IsNavigating || (message == null)) return;

                    var page = message.Content;
                    BiggestBoxCompositionHost.MefHost.NavigationTarget.SetNavigationContent(page);
                });
        }
    }
}
