﻿using System;
using System.Collections.Generic;
using System.Windows.Input;
using System.Xml.Linq;
using GalaSoft.MvvmLight.Command;
using GalaSoft.MvvmLight.Messaging;

namespace Songhay.Wpf.BiggestBox.ViewModels
{
    using Properties;
    using Songhay.Models;
    using Songhay.Mvvm.Models;
    using Songhay.Net;
    using Songhay.Wpf.Mvvm.ViewModels;
    using Songhay.Xml;

    /// <summary>
    /// Client View Model
    /// </summary>
    public class ClientViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="FlowViewModel"/> class.
        /// </summary>
        public ClientViewModel()
        {
            this.Title = Settings.Default["ApplicationTitle"].ToString();

            HttpVerbAsyncUtility.Get(new Uri(Settings.Default.ApplicationIndex),
                new Action<object, System.Net.DownloadStringCompletedEventArgs>((o, args) =>
                {
                    var xd = XDocument.Parse(args.Result);
                    var data = OpmlUtility.GetDocument(xd.Root, OpmlUtility.rx);

                    this.IndexCreatedOn = data.OpmlHead.DateCreated.Value;

                    this.Outlines = data.OpmlBody.Outlines;
                    this.IndexModifiedOn = data.OpmlHead.DateModified.Value;
                    this.IndexOwnerEmailAddress = data.OpmlHead.OwnerEmail;
                    this.IndexOwnerName = data.OpmlHead.OwnerName;
                    this.IndexTitle = data.OpmlHead.Title;
                }));

            #region commanding:

            this.IndexItemCommand = new RelayCommand<string>(param =>
            {
                if(this.CommandDictionary.ContainsKey(param))
                {
                    this.CommandDictionary[param].Invoke();
                    return;
                }

                var message = new LightNavigationMessage<string>(param, "IndexItemCommand");
                Messenger.Default.Send(message);
            });

            #endregion
        }

        /// <summary>
        /// Gets or sets the index created on.
        /// </summary>
        /// <value>The index created on.</value>
        public DateTime IndexCreatedOn
        {
            get { return this._indexCreatedOn; }
            set
            {
                this._indexCreatedOn = value;
                base.RaisePropertyChanged("IndexCreatedOn");
            }
        }

        /// <summary>
        /// Gets the index item command.
        /// </summary>
        /// <value>The index item command.</value>
        public ICommand IndexItemCommand { get; private set; }

        /// <summary>
        /// Gets or sets the index items.
        /// </summary>
        /// <value>The index items.</value>
        public IEnumerable<OpmlOutline> Outlines
        {
            get { return this._indexItems; }
            set
            {
                this._indexItems = value;
                base.RaisePropertyChanged("Outlines");
            }
        }

        /// <summary>
        /// Gets or sets the modified on.
        /// </summary>
        /// <value>The modified on.</value>
        public DateTime IndexModifiedOn
        {
            get { return this._indexModifiedOn; }
            set
            {
                this._indexModifiedOn = value;
                base.RaisePropertyChanged("IndexModifiedOn");
            }
        }

        /// <summary>
        /// Gets or sets the index owner email address.
        /// </summary>
        /// <value>The index owner email address.</value>
        public string IndexOwnerEmailAddress
        {
            get { return this._indexOwnerEmailAddress; }
            set
            {
                this._indexOwnerEmailAddress = value;
                base.RaisePropertyChanged("IndexOwnerEmailAddress");
            }
        }

        /// <summary>
        /// Gets or sets the name of the index owner.
        /// </summary>
        /// <value>The name of the index owner.</value>
        public string IndexOwnerName
        {
            get { return this._indexOwnerName; }
            set
            {
                this._indexOwnerName = value;
                base.RaisePropertyChanged("IndexOwnerName");
            }
        }

        /// <summary>
        /// Gets or sets the index title.
        /// </summary>
        /// <value>The index title.</value>
        public string IndexTitle
        {
            get { return this._indexTitle; }
            set
            {
                this._indexTitle = value;
                base.RaisePropertyChanged("IndexTitle");
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title { get; set; }

        DateTime _indexCreatedOn;
        IEnumerable<OpmlOutline> _indexItems;
        DateTime _indexModifiedOn;
        string _indexOwnerEmailAddress;
        string _indexOwnerName;
        string _indexTitle;
    }
}
