﻿using System;
using System.Security.Principal;

namespace Songhay.Wpf.BiggestBox.ViewModels
{
    using Properties;
    using Songhay.Extensions;
    using Songhay.Wpf.Mvvm.ViewModels;

    /// <summary>
    /// Defines the View Model for the Client header.
    /// </summary>
    public class HeaderViewModel : ViewModelBase
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="HeaderViewModel"/> class.
        /// </summary>
        public HeaderViewModel()
        {
            if(base.IsInDesignMode)
            {
                this.ServerName = "Server Name";
                this.ServerNameUnc = "\\\\ServerName";
                this.WindowsIdentityDisplayLine = "Windows Identity: DesignTime";
                this.Title = "Application Title";
            }
            else
            {
                this.ServerName = EnvironmentExtensions.GetConventionalValue("MachineName").ToUpper();
                this.ServerNameUnc = String.Format("\\\\{0}", this.ServerName);
                this.WindowsIdentityDisplayLine = string.Format("Windows Identity: {0}", WindowsIdentity.GetCurrent().Name);
                this.Title = Settings.Default["ApplicationTitle"].ToString();
            }
        }

        /// <summary>
        /// Gets or sets the name of the server.
        /// </summary>
        /// <value>The name of the server.</value>
        public string ServerName { get; set; }

        /// <summary>
        /// Gets or sets the server name in UNC format.
        /// </summary>
        /// <value>The server name (UNC).</value>
        public string ServerNameUnc { get; set; }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title { get; set; }

        /// <summary>
        /// Gets or sets the windows identity display line.
        /// </summary>
        /// <value>The windows identity display line.</value>
        public string WindowsIdentityDisplayLine { get; set; }
    }
}
