﻿using System;
using GalaSoft.MvvmLight.Messaging;

namespace Songhay.Mvvm.Models
{
    /// <summary>
    /// Defines the conventional MVVM Light Navigation Message.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class LightNavigationMessage<T> : GenericMessage<T>
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="LightNavigationMessage&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="messageContent">Content of the message.</param>
        public LightNavigationMessage(T messageContent)
            : base(messageContent)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LightNavigationMessage&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="messageContent">Content of the message.</param>
        public LightNavigationMessage(string location, T messageContent)
            : base(messageContent)
        {
            this.HasWellFormedUriString = Uri.IsWellFormedUriString(location, UriKind.Absolute);
            this.TargetAbsoluteUri = this.HasWellFormedUriString ? new Uri(location, UriKind.Absolute) : null;
            this.TargetLocation = this.HasWellFormedUriString ? null : location;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LightNavigationMessage&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="messageSender">The message sender.</param>
        /// <param name="messageContent">Content of the message.</param>
        public LightNavigationMessage(string location, object messageSender, T messageContent)
            : base(messageSender, messageContent)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LightNavigationMessage&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <param name="messageSender">The message sender.</param>
        /// <param name="messageTarget">The message target.</param>
        /// <param name="messageContent">Content of the message.</param>
        public LightNavigationMessage(string location, object messageSender, object messageTarget, T messageContent)
            : base(messageSender, messageTarget, messageContent)
        {
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance has a well-formed URI string.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance has a well-formed URI string; otherwise, <c>false</c>.
        /// </value>
        public bool HasWellFormedUriString { get; private set; }

        /// <summary>
        /// Gets or sets the source.
        /// </summary>
        /// <value>The source.</value>
        public LightMessageSource Source { get; set; }

        /// <summary>
        /// Gets or sets the view id.
        /// </summary>
        /// <value>The view id.</value>
        public string ViewId { get; set; }

        /// <summary>
        /// Gets or sets the target location.
        /// </summary>
        /// <value>The target location.</value>
        public string TargetLocation { get; set; }

        /// <summary>
        /// Gets or sets the target absolute URI.
        /// </summary>
        /// <value>The target absolute URI.</value>
        public Uri TargetAbsoluteUri { get; set; }
    }
}
