﻿using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;

namespace Songhay.Extensions
{
    /// <summary>
    /// Extensions of <see cref="System.Xml.Linq.XElement"/>.
    /// </summary>
    public static class XElementExtensions
    {
        /// <summary>
        /// Returns the attribute value or null.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <param name="attributeName">Name of the attribute.</param>
        public static string AttributeValueOrNull(this XElement element, string attributeName)
        {
            if(element == null) return null;

            string s = null;
            var attr = element.Attribute(attributeName);
            if(attr != null) s = attr.Value;
            return s;
        }

        /// <summary>
        /// Returns the element value or null.
        /// </summary>
        /// <param name="elements">The elements.</param>
        public static string ElementValueOrNull(this IEnumerable<XElement> elements)
        {
            string s = null;
            var element = elements.FirstOrDefault();
            if(element != null) s = element.Value;
            return s;
        }

        /// <summary>
        /// Gets the <see cref="System.Xml.Linq.XElement"/>.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="name">The name.</param>
        public static XElement GetElement(this XNode node, XName name)
        {
            var element = node as XElement;
            if((element != null) && (element.Name == name))
            {
                return element;
            }
            return null;
        }

        /// <summary>
        /// Determines whether the <see cref="System.Xml.Linq.XElement"/>
        /// has the specified <see cref="System.Xml.Linq.XName"/>.
        /// </summary>
        /// <param name="element">The element.</param>
        /// <param name="name">The name.</param>
        /// <returns>
        /// 	<c>true</c> when the element has the name; otherwise, <c>false</c>.
        /// </returns>
        public static bool HasElementName(this XElement element, XName name)
        {
            if(element == null) return false;
            return (element.Name == name);
        }

        /// <summary>
        /// Determines whether the <see cref="System.Xml.Linq.XNode"/>
        /// has the specified <see cref="System.Xml.Linq.XName"/>.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="name">The name.</param>
        /// <returns>
        /// 	<c>true</c> when the node has the name; otherwise, <c>false</c>.
        /// </returns>
        public static bool HasElementName(this XNode node, XName name)
        {
            var element = node as XElement;
            return (element != null) && (element.Name == name);
        }

        /// <summary>
        /// Determines whether the specified node is <see cref="System.Xml.Linq.XElement"/>.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <returns>
        /// 	<c>true</c> if the specified node is element; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsElement(this XNode node)
        {
            var element = node as XElement;
            return (element != null);
        }
    }
}
